import logging

from aws_ir.libs import compromised
from aws_ir.libs import connection
from aws_ir.libs import plugin

from aws_ir.plans import steps_to_list

logger = logging.getLogger(__name__)


"""Compromise class for Key Compromise Procedure"""


class Compromise(object):

    def __init__(
        self,
        examiner_cidr_range='0.0.0.0/0',
        compromised_access_key_id=None,
        region='us-west-2',
        case=None,
        steps=None
    ):

        if compromised_access_key_id is None:
            raise ValueError(
                'Must specify an access_key_id for the compromised key.'
            )

        self.case_type = 'Key'
        self.compromised_access_key_id = compromised_access_key_id
        self.region = region
        self.case = case
        self.plugins = plugin.Core()
        self.steps = steps_to_list(steps)

    def mitigate(self):
        """Any steps that run as part of key compromises."""
        access_key = self.compromised_access_key_id
        compromised_resource = compromised.CompromisedMetadata(
            compromised_object_inventory={
                'access_key_id': access_key,
                'region': self.region
            },
            case_number=self.case.case_number,
            type_of_compromise='key_compromise'
        ).data()

        session = connection.Connection(
            type='session',
            region='us-west-2'
        ).connect()

        logger.info("Attempting key disable.")

        for action in self.steps:
            step = self.plugins.source.load_plugin(action)
            step.Plugin(
                boto_session=session,
                compromised_resource=compromised_resource,
                dry_run=False
            )



import aws_ir_plugins
import os


from pluginbase import PluginBase


class Core(object):
    """Enumerates core plugins that are part of the AWS_IR offering."""
    def __init__(self):
        self.here = os.path.abspath(os.path.dirname(__file__))

        self.plugin_base = PluginBase(
            package='aws_ir.plugins',
            searchpath=[
                os.path.dirname(aws_ir_plugins.__file__),
                (os.getenv("HOME") + '/.awsir/plugins')
            ]
        )

        self.source = self.plugin_base.make_plugin_source(
            searchpath=[
                os.path.dirname(aws_ir_plugins.__file__),
                (os.getenv("HOME") + '/.awsir/plugins')
            ]
        )

        self.list = self.source.list_plugins()

    def key_plugins(self):
        """Return list of only the plugins that relate to the access key."""
        plugins = ""
        for p in self.list:
            if "_key" in p:
                if plugins == "":
                    plugins = p
                else:
                    plugins = plugins + ',' + p
        return plugins

    def instance_plugins(self):
        """Return list of only the plugins that relate to the instance compromise."""
        plugins = ""
        for p in self.list:
            if "_host" in p:
                if plugins == "":
                    plugins = p
                else:
                    plugins = plugins + ',' + p
        plugins = plugins + ',' + 'get_memory'
        return plugins

    def lambda_plugins(self):
        """Return list of only the plugins that relate to the lambda compromise."""
        plugins = ""
        for p in self.list:
            if "_lambda" in p:
                if plugins == "":
                    plugins = p
                else:
                    plugins = plugins + ',' + p
        return plugins



"""Case management operations for AWS_IR."""
import logging
import os
import random
import sys

from datetime import datetime

import aws_ir
from aws_ir.libs import aws
from aws_ir.libs import connection
from aws_ir.libs import inventory
from aws_ir.libs import s3bucket


logger = logging.getLogger(__name__)


class Case(object):
    """Takes case number and examiner cidr."""
    def __init__(
        self,
        case_number=None,
        examiner_cidr_range='0.0.0.0/0',
        case_bucket=None,
        profile='default',
        case_type=None
    ):
        self.profile = profile

        self.ec2_client = connection.Connection(
            type='client',
            service='ec2',
            region='us-west-2',
            profile=self.profile
        )

        self.s3_resource = connection.Connection(
            type='resource',
            service='s3',
            region='us-west-2',
            profile=self.profile
        )

        if case_number:
            self.case_number = case_number
        else:
            self.case_number = self._generate_case_number()

        if case_bucket:
            self.case_bucket = case_bucket
        else:
            self.case_bucket = self._setup_bucket(region='us-west-2')

        self.examiner_cidr_range = examiner_cidr_range

    def prep_aws_connections(self):
        """Get all the required information before doing the mitigation. """
        logger.info("Initial connection to AmazonWebServices made.")

        self.amazon = aws.AmazonWebServices(
            self.ec2_client
        )

        self.available_regions = self.amazon.regions

        logger.info(("Inventory AWS Regions Complete {region_count} "
                     "found.".format(region_count=len(self.amazon.regions))))

        self.availability_zones = self.amazon.availability_zones

        logger.info(
            (
                "Inventory Availability Zones Complete {zone_count} "
                "found.".format(zone_count=len(self.amazon.availability_zones))
            )
        )

        logger.info(("Beginning inventory of resources world wide.  "
                     "This might take a minute..."))

        self.aws_inventory = inventory.Inventory(
            self.ec2_client,
            self.available_regions
        )

        logger.info(("Inventory complete.  Proceeding to resource "
                     "identification."))

        self.inventory = self.aws_inventory.inventory

    def _rename_log_file(self, case_number, resource_id, base_dir="/tmp"):
        """Move all log files to standard naming format"""
        try:
            os.rename(
                ("{base_dir}/{case_number}-aws_ir.log").format(
                    base_dir=base_dir,
                    case_number=case_number,
                ),
                ("{base_dir}/{case_number}-{resource_id}-aws_ir.log").format(
                    base_dir=base_dir,
                    case_number=case_number,
                    resource_id=resource_id
                )
            )
            return True
        except Exception:
            return False

    def copy_logs_to_s3(self, base_dir="/tmp"):
        """Convinience function to put all case logs to s3 at the end"""
        case_bucket = self._get_case_bucket()
        logs = self._get_case_logs(base_dir=base_dir)
        for log in logs:
            case_bucket.upload_file(
                "{base_dir}/{log}".format(
                    base_dir=base_dir,
                    log=log
                ),
                log
            )

    def teardown(self, region, resource_id):
        """Any final post mitigation steps universal to all plans. """
        try:
            aws_ir.wrap_log_file(self.case_number)
            self._rename_log_file(self.case_number, resource_id)
            self.copy_logs_to_s3()
            processing_end_messaging = (
                """Processing complete for {case_number}\n"""
                """Artifacts stored in s3://{case_bucket}"""
            ).format(case_number=self.case_number,
                     case_bucket=self.case_bucket)
            print(processing_end_messaging)
            sys.exit(0)
        except Exception as e:
            logger.error(
                ("Error uploading case logs for {case_number} to s3 "
                 "bucket {case_bucket}: {ex}".format(
                     case_number=self.case_number,
                     case_bucket=self.case_bucket,
                     ex=e)
                 )
            )

            sys.exit(1)

    def _get_case_logs(self, base_dir="/tmp"):
        """Enumerates all case logs based on case number from system /tmp"""
        files = []
        for file in os.listdir(base_dir):
            if file.startswith(self.case_number):
                files.append(file)
        return files

    def _setup_bucket(self, region):
        """Wrap s3 find or create in object"""
        client = connection.Connection(
            type='client',
            service='s3'
        ).connect()

        bucket_name = s3bucket.CaseBucket(
            self.case_number,
            region,
            client,
            self.s3_resource
        ).bucket.name

        return bucket_name

    def _get_case_bucket(self):
        return self.s3_resource.connect().Bucket(self.case_bucket)

    def _generate_case_number(self):
        return datetime.utcnow().strftime(
            'cr-%y-%m%d%H-{0:04x}'
        ).format(
            random.randint(0, 2 ** 16)
        )



from datetime import datetime
import logging
import os
import time

from aws_ir import _version

__version__ = _version.__version__


def set_stream_logger(name="aws_ir", level=logging.INFO,
                      format_string=None):

    if format_string is None:
        format_string = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    time_format = "%Y-%m-%dT%H:%M:%S"

    logger = logging.getLogger(name)
    logger.setLevel(level)
    streamHandler = logging.StreamHandler()
    streamHandler.setLevel(level)
    streamFormatter = logging.Formatter(format_string, time_format)
    streamHandler.setFormatter(streamFormatter)
    logger.addHandler(streamHandler)


def set_file_logger(case_number, name="aws_ir", level=logging.INFO,
                    base_dir="/tmp", desc="AWS_IR Action"):
    log_file = "{base_dir}/{case_number}-aws_ir.log".format(
        base_dir=base_dir, case_number=case_number
    )

    logger = logging.getLogger(name)
    logger.setLevel(level)
    fileHandler = logging.FileHandler(log_file, mode='a')
    fileHandler.setLevel(level)
    fileFormatter = logging.Formatter(
        "\t{'timestamp': %(unixtime)s, 'message': '%(message)s', " +
        "desc: '{desc}', 'datetime': '%(isotime)s'}},".format(desc=desc)
    )
    fileHandler.setFormatter(fileFormatter)
    logger.addHandler(fileHandler)


def wrap_log_file(case_number, base_dir="/tmp"):
    log_file = "{base_dir}/{case_number}-aws_ir.log".format(
        base_dir=base_dir, case_number=case_number
    )
    # if log_file exists and is not empty  append a closing "]" to the file
    if os.path.isfile(log_file) and (os.path.getsize(log_file) > 0):
        with open(log_file, 'a') as f:
            f.write("]")
            f.flush()
            f.close()
    # otherwise write an opening "[" to the file
    else:
        with open(log_file, 'w+') as f:
            f.write("[\n")
            f.flush()
            f.close()


class TimesketchLogger(logging.getLoggerClass()):

    def __init__(self, *args, **kwargs):
        super(TimesketchLogger, self).__init__(*args, **kwargs)

    def _log(self, level, msg, args, exc_info=None, extra=None):
        super(TimesketchLogger, self)._log(level, msg, args, exc_info=exc_info,
                                           extra=self.__get_times())

    def __get_times(self):
        tm = int(time.time())
        dt = datetime.utcfromtimestamp(tm).isoformat()
        times = {'unixtime': tm, 'isotime': dt}
        return times


class NullHandler(logging.Handler):
    def emit(self, record):
        pass


logging.setLoggerClass(TimesketchLogger)

logging.getLogger('aws_ir').addHandler(NullHandler())



import boto3
import logging

logger = logging.getLogger(__name__)


class Connection(object):
    def __init__(self, type, service=None, region='us-west-2', profile='default'):
        self.region = region
        self.connection_type = type
        self.service = service
        self.client = None
        self.resource = None
        self.profile = profile
        try:
            boto3.setup_default_session(profile_name=self.profile)
        except Exception as e:
            logger.info("Problem setting default boto3 session: {}".format(e))

    def connect(self):
        if self.connection_type is None:
            raise AttributeError(
                "Could not determine connect type.  Set client or resource."
            )
        elif self.connection_type == "client":
            client = boto3.client(
                self.service,
                region_name=self.region
            )
            self.client = client
            return self.client
        elif self.connection_type == "resource":
            resource = boto3.resource(
                self.service,
                region_name=self.region
            )
            self.resource = resource
            return self.resource
        elif self.connection_type == "session":
            try:
                session = boto3.Session(
                    region_name=self.region,
                    profile_name=self.profile
                )
                logger.info(
                    "Returning session for default profile."
                )
            except Exception as e:
                logger.info(
                    "We are likely running on AWS instance.: {}".format(e)
                )
                session = boto3.Session(
                    region_name=self.region
                )
            return session
        else:
            raise AttributeError(
                "Connection type is not supported."
            )



#!/usr/bin/env python3
import argparse
import logging
import os
import sys

import aws_ir
from aws_ir import __version__
from aws_ir.libs import case
from aws_ir.libs import plugin

# Support for multiple incident plans coming soon
from aws_ir.plans import host
from aws_ir.plans import key


"""Basic arg parser for AWS_IR cli"""


class cli():
    def __init__(self):
        self.config = None
        self.prog = sys.argv[0].split('/')[-1]

    """Parent parser for top level flags"""
    def parse_args(self, args):

        parser = argparse.ArgumentParser(
            description="""
                Incident Response command line for Amazon Web Services.
                This command line interface is designed to process host and key
                based incursions without delay or error.
            """
        )

        optional_args = parser.add_argument_group()

        optional_args.add_argument(
            '--version',
            action='version',
            version="%(prog)s {ver}".format(ver=__version__))

        optional_args.add_argument(
            '--verbose',
            action='store_true',
            help='log debug messages')

        optional_args.add_argument(
            '--profile',
            default='default',
            help="""
                A named boto profile to use instead of the default profile.
                """
        )

        optional_args.add_argument(
            '--case-number',
            default=None,
            help="""
                The case number to use., usually of the form
                "cr-16-053018-2d2d"
                """
        )

        optional_args.add_argument(
            '--examiner-cidr-range',
            default='0.0.0.0/0',
            help="""
                The IP/CIDR for the examiner and/or the tool.
                This will be added as the only allowed range
                in the isolated security group.
            """
        )

        optional_args.add_argument(
            '--bucket-name',
            default=None,
            help="""
                Optional.
                The id of the s3 bucket to use.
                This must already exist
            """
        )

        optional_args.add_argument(
            '--dry-run',
            action='store_true',
            help="""
                Dry run. Pass dry run
                parameter to perform API calls
                but will not modify any resources.
            """
        )

        subparsers = parser.add_subparsers(dest="compromise-type")
        subparsers.required = True

        instance_compromise_parser = subparsers.add_parser(
            'instance-compromise', help=''
        )

        instance_compromise_parser.add_argument(
            '--target',
            required=False,
            help="""
                instance-id|instance-ip
            """
        )

        instance_compromise_parser.add_argument(
            '--targets',
            required=False,
            help="""
                File of resources to process instance-id or ip-address.
            """
        )

        instance_compromise_parser.add_argument(
            '--user',
            required=False,
            help="""
                this is the privileged ssh user
                for acquiring memory from the instance.
                Required for memory only.
            """
        )
        instance_compromise_parser.add_argument(
            '--ssh-key',
            required=False,
            help='provide the path to the ssh private key for the user. Required for memory only.'
        )

        instance_compromise_parser.add_argument(
            '--plugins',
            required=False,
            default="gather_host,isolate_host,"
                    "tag_host,snapshotdisks_host,"
                    "examineracl_host,get_memory,stop_host",
            help="Run some or all of the plugins in a custom order. "
                 "Provided as a comma separated list of "
                 "supported plugins: \n"
                 "{p}".format(
                    p=plugin.Core().instance_plugins()
                 )
        )

        instance_compromise_parser.set_defaults(func="instance_compromise")

        key_compromise_parser = subparsers.add_parser(
            'key-compromise',
            help=''
        )

        key_compromise_parser.add_argument(
            '--access-key-id', required=True, help=''
        )

        key_compromise_parser.add_argument(
            '--plugins',
            default="disableaccess_key,revokests_key",
            required=False,
            help="Run some or all of the plugins in a custom order."
                 " Provided as a comma separated list"
                 "Supported plugins: \n"
                 "{p}".format(
                    p=plugin.Core().key_plugins()
                 )
        )

        key_compromise_parser.set_defaults(func="key_compromise")

        return parser.parse_args(args)

    """Logic to decide on host or key compromise"""
    def run(self):
        self.config = self.parse_args(sys.argv[1:])
        case_obj = case.Case(
            self.config.case_number,
            self.config.examiner_cidr_range,
            self.config.bucket_name,
            self.config.profile
        )

        if self.config.verbose:
            log_level = logging.DEBUG
        else:
            log_level = logging.INFO

        aws_ir.set_stream_logger(level=log_level)
        aws_ir.set_file_logger(case_obj.case_number, level=log_level)
        logger = logging.getLogger(__name__)

        aws_ir.wrap_log_file(case_obj.case_number)
        logger.info("Initialization successful proceeding to incident plan.")
        if self.config.func == 'instance_compromise':
            if self.config.target:
                case_obj.prep_aws_connections()
                hc = host.Compromise(
                    user=self.config.user,
                    ssh_key_file=self.config.ssh_key,
                    target=self.config.target,
                    prog=self.prog,
                    case=case_obj,
                    steps=self.config.plugins
                )
                try:
                    hc.mitigate()
                except KeyboardInterrupt:
                    pass
            if self.config.targets:
                case_obj.prep_aws_connections()
                logger.info(
                    'Alert : multi-host mode engaged targets in file will attempt processing.'
                )
                batch_file = os.path.abspath(self.config.targets)

                with open(batch_file) as f:
                    targets = f.read().split('\n')

                for target in targets:
                    if target is not '':
                        hc = host.Compromise(
                            user=self.config.user,
                            ssh_key_file=self.config.ssh_key,
                            target=target,
                            prog=self.prog,
                            case=case_obj,
                            steps=self.config.plugins
                        )
                        try:
                            logger.info("Attempting processing instance {i}".format(i=target))
                            hc.mitigate()
                        except KeyboardInterrupt:
                            pass
        elif self.config.func == 'key_compromise':
            kc = key.Compromise(
                examiner_cidr_range=self.config.examiner_cidr_range,
                compromised_access_key_id=self.config.access_key_id,
                region='us-west-2',
                case=case_obj,
                steps=self.config.plugins
            )

            try:
                kc.mitigate()
            except KeyboardInterrupt:
                pass


if __name__ == '__main__':
    c = cli()
    if c.prog is not None:
        c.run()




def steps_to_list(string_literal):
    """Takes a comma separated list and returns a list data type."""
    new_list = []

    for item in string_literal.split(','):
        new_list.append(item)

    return new_list



import logging
import margaritashotgun
import platform

from margaritashotgun.repository import Repository

logger = logging.getLogger(__name__)


class Memory(object):
    def __init__(
        self,
        compromised_resource,
        dry_run,
        boto_session
    ):

        self.session = boto_session
        self.compromised_resource = compromised_resource
        self.compromise_type = compromised_resource['compromise_type']
        self.dry_run = dry_run

        # Check if logging level is set to logging.DEBUG
        if logger.getEffectiveLevel() < logging.INFO:
            self.verbose = True
        else:
            self.verbose = False

    def get_memory(
        self,
        bucket,
        ip,
        user,
        key,
        case_number,
        port=None,
        password=None
    ):
        name = 'margaritashotgun'
        config = dict(
            aws=dict(bucket=bucket),
            hosts=[
                dict(
                    addr=ip, port=port,
                    username=user,
                    password=password,
                    key=key
                )
            ],
            workers='auto',
            logging={
                    'dir': '/tmp/',
                    'prefix': ("{case_number}-{ip}").format(
                        ip=ip,
                        case_number=case_number
                    )
            },
            repository=dict(
                enabled=True,
                url=('https://threatresponse-lime'
                     '-modules.s3.amazonaws.com/')
            )
        )
        if 'Darwin' in platform.system():
            config['repository']['gpg_verify'] = False
        capture_client = margaritashotgun.client(
            name=name,
            config=config,
            library=True,
            verbose=self.verbose
        )

        try:
            repository_url = 'https://threatresponse-lime-modules.s3.amazonaws.com/'
            if 'Darwin' in platform.system():
                repository_gpg_verify = False
            else:
                repository_gpg_verify = True

            self.repo = Repository(repository_url, repository_gpg_verify)

            if 'Darwin' not in platform.system():
                self.repo.init_gpg()

            return capture_client.run()
        except Exception:
            if 'Darwin' not in platform.system():
                logger.critical("GPG key not in trust chain attempting interactive installation.")
                self.repo.prompt_for_install()
            return capture_client.run()



import re
from distutils.command.build import build
from setuptools import setup

VERSION = re.search(
    r"^__version__ = ['\"]([^'\"]*)['\"]",
    open('aws_ir/_version.py', 'r').read(),
    re.MULTILINE
).group(1)

setup(name="aws_ir",
      version=VERSION,
      author="Andrew Krug, Alex McCormack, Joel Ferrier, Jeff Parr",
      author_email="andrewkrug@gmail.com,developer@amccormack.net,joel@ferrier.io",
      packages=["aws_ir", "aws_ir/libs", "aws_ir/plans"],
      license="MIT",
      description="AWS Incident Response ToolKit",
      scripts=['bin/aws_ir'],
      url='https://github.com/ThreatResponse/aws_ir',
      download_url="https://github.com/ThreatResponse/aws_ir/archive/v0.3.0.tar.gz",
      use_2to3=True,
      zip_safe=True,
      install_requires=['boto3>=1.3.0',
                        'progressbar_latest',
                        'logutils==0.3.3',
                        'requests',
                        'structlog',
                        'pytz',
                        'jinja2',
                        'pluginbase',
                        'margaritashotgun>=0.4.1',
                        'aws-ir-plugins>=0.0.3'
                        ],
      setup_requires=['pytest-runner'],
      tests_require=['pytest',
                     'pytest-cov',
                     'pytest-watch',
                     'moto',
                     'mock'],
      )



import logging

logger = logging.getLogger(__name__)


class Query(object):
    def __init__(self, client, regions, inventory_type='aws'):
        self.client = client
        self.regions = regions
        self.type = inventory_type
        self.result = self.get_all_running_from_aws()

    def get_running_by_region_aws(self, region):
        inventory = []
        self.client.region = region
        client = self.client.connect()

        reservations = client.describe_instances(
            Filters=[{'Name': 'instance-state-name', 'Values': ['running']}]
        )['Reservations']

        for reservation in reservations:
            for instance in reservation['Instances']:
                instance_data = self._extract_data_aws(instance, region)
                instance_data['region'] = region
                inventory.append(instance_data)
        return inventory

    def _extract_data_aws(self, instance, region):
        return dict(
            public_ip_address=instance.get('PublicIpAddress', None),
            private_ip_address=instance.get('PrivateIpAddress', None),
            instance_id=instance['InstanceId'],
            launch_time=instance['LaunchTime'],
            platform=instance.get('Platform', None),
            vpc_id=instance['VpcId'],
            ami_id=instance['ImageId'],
            volume_ids=[
                bdm['Ebs']['VolumeId']
                for bdm in instance.get(
                    'BlockDeviceMappings', []
                )
            ],
            region=region
        )

    def get_all_running_from_aws(self):
        inventory = {}
        for region in self.regions:
            logger.info(("Searching {region} for instance.".format(region=region)))
            inventory[region] = self.get_running_by_region_aws(region)
        return inventory


class InventoryType(object):
    def __init__(self):
        self.type = self._inventory_type()

    def _inventory_type(self):
        return "aws"


class Inventory(object):
    def __init__(self, client, regions):
        self.client = client
        self.regions = regions
        self.type = 'aws'

        self.inventory = Query(
            self.client,
            self.regions,
            self.type
        ).result

    def locate_instance_by_ip(self, ip):
        for region in self.regions:
            located = list(
                filter(
                    lambda x: x['public_ip_address'] == ip,
                    self.inventory[region]
                )
            )

            if len(located) == 0:
                pass
            else:
                return located[0]

    def locate_instance_by_id(self, instance_id):
        for region in self.regions:
            located = list(
                filter(
                    lambda x: x['instance_id'] == instance_id,
                    self.inventory[region]
                )
            )

            if len(located) == 0:
                pass
            else:
                return located[0]



import logging

from aws_ir.libs import compromised
from aws_ir.libs import connection
from aws_ir.libs import plugin
from aws_ir.libs import volatile

from aws_ir.plans import steps_to_list


logger = logging.getLogger(__name__)


"""Compromise class for Host Compromise"""


class Compromise(object):
    """Procedures for responding to a HostCompromise.

    ** For now, assume only Linux system.
    """
    def __init__(
            self,
            user=None,
            ssh_key_file=None,
            target=None,
            prog=None,
            case=None,
            steps=None
    ):

        if target is None:
            raise ValueError(
                'You must specify an ip-address or instance-id.'
            )

        self.prog = prog
        self.ssh_key_file_path = ssh_key_file
        self.user = user
        self.case_type = 'Host'

        self.target = target
        self.case = case

        self.plugins = plugin.Core()
        self.steps = steps_to_list(steps)

    def _target_type(self):
        """Returns the target type based on regex."""
        if len(self.target.split('.')) is 4:
            return 'ip-address'
        else:
            return 'instance-id'

    def mitigate(self):

        if self._target_type() == 'ip-address':
            search = self.case.aws_inventory.locate_instance_by_ip(
                self.target
            )
        if self._target_type() == 'instance-id':
            search = self.case.aws_inventory.locate_instance_by_id(
                self.target
            )

        if search is None:
            raise ValueError(
                'Target ip-address or instance-id not found in inventory.'
            )

        compromised_resource = compromised.CompromisedMetadata(
            compromised_object_inventory=search,
            case_number=self.case.case_number,
            type_of_compromise='host_compromise',
            examiner_cidr_range=self.case.examiner_cidr_range
        ).data()

        session = connection.Connection(
            type='session',
            region=compromised_resource['region']
        ).connect()

        logger.info(
            "Proceeding with incident plan steps included are {steps}".format(steps=self.steps)
        )

        for action in self.steps:
            logger.info("Executing step {step}.".format(step=action))
            if 'get_memory' not in action:
                step = self.plugins.source.load_plugin(action)
                step.Plugin(
                    boto_session=session,
                    compromised_resource=compromised_resource,
                    dry_run=False
                )
            elif 'get_memory' == action:
                logger.info("attempting memory run")
                self.do_mem(session, compromised_resource)

    def do_mem(self, session, compromised_resource):
        if compromised_resource['platform'] == 'windows':
            logger.info('Platform is Windows skipping live memory')
        elif self.case.examiner_cidr_range == '0.0.0.0/0':
            logger.info(
                "Examiner CIDR not provided skipping memory acquisition."
            )
        else:
            logger.info(
                (
                    "Attempting run margarita shotgun for {user} on {ip} with {keyfile}".format(
                        user=self.user,
                        ip=compromised_resource.get('public_ip_address', None),
                        keyfile=self.ssh_key_file_path
                    )
                )
            )

            try:
                volatile_data = volatile.Memory(
                    boto_session=session,
                    compromised_resource=compromised_resource,
                    dry_run=False
                )

                results = volatile_data.get_memory(
                    bucket=self.case.case_bucket,
                    ip=compromised_resource['public_ip_address'],
                    user=self.user,
                    key=self.ssh_key_file_path,
                    case_number=self.case.case_number
                )

                print(results)

                logger.info(
                    (
                        "memory capture completed for: {0}, "
                        "failed for: {1} ".format(
                            results['completed'],
                            results['failed']
                        )
                    )
                )
            except Exception as ex:
                # raise keyboard interrupt passed during memory capture
                if isinstance(ex, KeyboardInterrupt):
                    raise
                else:
                    logger.error(("Memory acquisition failure with exception "
                                  "{exception}. ".format(exception=ex)))
